"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.modify = exports.createModifiedReadableStream = exports.createModifiedReadable = exports.initialize = void 0;
const univ_conv_1 = require("univ-conv");
let _createModifiedReadable;
let _createModifiedReadableStream;
let initialized = false;
async function initialize() {
    if (initialized) {
        return;
    }
    initialized = true;
    if (univ_conv_1.hasReadable) {
        _createModifiedReadable = (await Promise.resolve().then(() => require("./mods-node"))).default;
    }
    if (univ_conv_1.hasReadableStream) {
        _createModifiedReadableStream = (await Promise.resolve().then(() => require("./mods-web"))).default;
    }
}
exports.initialize = initialize;
const createModifiedReadable = async (src, mods) => {
    await initialize();
    if (_createModifiedReadable) {
        return _createModifiedReadable(src, mods);
    }
    throw new Error("createModifiedReadable is undefined");
};
exports.createModifiedReadable = createModifiedReadable;
const createModifiedReadableStream = async (src, mods) => {
    await initialize();
    if (_createModifiedReadableStream) {
        return _createModifiedReadableStream(src, mods);
    }
    throw new Error("createModifiedReadableStream is undefined");
};
exports.createModifiedReadableStream = createModifiedReadableStream;
async function modify(src, ...mods) {
    const conv = await (0, univ_conv_1.getUnivConv)();
    const u8 = await conv.convert("uint8array", src);
    const size = u8.length;
    for (const mod of mods) {
        const start = mod.start ?? 0;
        if (size <= start) {
            continue;
        }
        let length;
        if (mod.length == null) {
            length = size - start;
        }
        else {
            length = mod.length;
            if (size < start + length) {
                length = size - start;
            }
        }
        const data = await conv.convert("uint8array", mod.data, { length });
        u8.set(data, start);
    }
    return u8;
}
exports.modify = modify;
//# sourceMappingURL=mods.js.map