import { closeStream, getUnivConv } from "univ-conv";
async function createModifiedReadableStream(src, ...mods) {
    const conv = await getUnivConv();
    const reader = src.getReader();
    return new ReadableStream({
        start: async (controller) => {
            let iStart = 0;
            let done;
            do {
                const res = await reader.read();
                const value = res.value;
                done = res.done;
                if (!done) {
                    const chunk = value;
                    const size = chunk.byteLength;
                    try {
                        const iEnd = iStart + size;
                        for (const mod of mods) {
                            const mStart = mod.start ?? 0;
                            const mEnd = mStart + (mod.length ?? Number.MAX_SAFE_INTEGER);
                            if (mStart <= iStart && iEnd <= mEnd) {
                                const start = iStart - mStart;
                                const length = size;
                                controller.enqueue(conv.slice(mod.data, { start, length }));
                                return;
                            }
                            else if (iStart <= mStart && mStart <= iEnd) {
                                const chunkLen = mStart - iStart;
                                if (0 < chunkLen) {
                                    controller.enqueue(chunk.slice(0, chunkLen));
                                }
                                if (mEnd < iEnd) {
                                    const modLen = mEnd - mStart;
                                    controller.enqueue(conv.slice(mod.data, {
                                        start: 0,
                                        length: modLen,
                                    }));
                                    if (mEnd < iEnd) {
                                        const start = mEnd - iStart;
                                        const length = iEnd - mEnd;
                                        controller.enqueue(chunk.slice(start, length));
                                    }
                                }
                                return;
                            }
                            else if (mStart < iStart && iStart < mEnd) {
                                const start = iStart - mStart;
                                let length;
                                if (mEnd < iEnd) {
                                    length = mEnd - iStart;
                                }
                                else {
                                    length = iEnd - iStart;
                                }
                                controller.enqueue(conv.slice(mod.data, { start, length }));
                                return;
                            }
                            controller.enqueue(chunk);
                        }
                    }
                    finally {
                        iStart += size;
                    }
                }
            } while (!done);
            controller.close();
            reader.releaseLock();
            closeStream(src);
        },
        cancel: (e) => {
            reader.releaseLock();
            closeStream(src, e);
        },
    });
}
export default createModifiedReadableStream;
//# sourceMappingURL=mods-web.js.map